@@ -1,10 +1,12 @@
 #!/usr/bin/env python3
 # Author: Yao Yan
-# Description: This script is used to evaluate the annotation by participants with
-# gold standards, we will conduct the annotation based on subcategory: date, person name,
-# physical address
+# Description: This script is used to evaluate the annotation by participants
+# with gold standards, we will conduct the evaluation based on subcategory:
+# date, person name,physical address
+
 from abc import ABCMeta
 import json
+import os
 import re
 
 
@@ -24,12 +26,38 @@ class Evaluation(metaclass=ABCMeta):
     def __init__(self):
         pass
 
+    def convert_annotations(self, annotations):
+        if self.evaluation_type == "date":
+            annotation_key = "date_annotations"
+            post_path = "textDateAnnotations"
+        elif self.evaluation_type == "person":
+            annotation_key = "person_name_annotations"
+            post_path = "textPersonNameAnnotations"
+        elif self.evaluation_type == "address":
+            annotation_key = "physical_location_annotations"
+            post_path = "textPhysicalAddressAnnotations"
+        else:
+            raise ValueError("Must specify evaluation_type attribute")
+
+        all_annotations = []
+        for annotation in annotations:
+            # print(annotation)
+            noteid = annotation['annotationSource']['resourceSource']['name']
+            for annots in annotation[post_path]:
+                annots['noteId'] = os.path.basename(noteid)
+                all_annotations.append(annots)
+
+        new_annotations = {annotation_key: all_annotations}
+        return new_annotations
+
     def convert_dict(self, sys_file, gs_file):
         with open(gs_file) as f:
             gs = json.load(f)
+            gs = self.convert_annotations(gs)
             gs = gs[self.col]
         with open(sys_file) as f:
             sys = json.load(f)
+            sys = self.convert_annotations(sys)
             sys = sys[self.col]
 
         self.sys_dict_seq = self.json_dict_seq(sys)
@@ -50,8 +78,9 @@ class Evaluation(metaclass=ABCMeta):
             data_loc = '{}-{}'.format(noteId, start)
             text = anno['text']
             length = anno['length']
-            dateFormat = anno[self.annotation]
-            date_list = [text, dateFormat, length]
+            # dateFormat, personType, addressType
+            annotation_format = anno.get(self.annotation, float('nan'))
+            date_list = [text, annotation_format, length]
             json_dict[data_loc] = date_list
         return json_dict
 
@@ -65,14 +94,15 @@ class Evaluation(metaclass=ABCMeta):
             noteId = anno['noteId']
             start = anno['start']
             text = anno['text']
-            dateFormat = anno[self.annotation]
+            # dateFormat, personType, addressType
+            annotation_format = anno.get(self.annotation, float('nan'))
             sub_text = re.split(r'\s+', text)
             for sub in sub_text:
                 leng = len(sub)
                 data_loc = '{}-{}-{}'.format(noteId, start, leng)
                 start = start + leng + 1
                 # [text, dateFormat,length]
-                date_list = [sub, dateFormat, leng]
+                date_list = [sub, annotation_format, leng]
                 json_dict[data_loc] = date_list
         return json_dict
 
@@ -174,12 +204,20 @@ class Evaluation(metaclass=ABCMeta):
         self.print_out(tp, fp, fn, "token", "strict")
 
     def print_out(self, tp, fp, fn, type_up, type_lower):
-        # precision (P): TP / (TP + FP)
-        # Recall (R): TP / (TP + FN)
-        # F1 score: 2 * ((P * R) / (P + R))
-        precision = round(tp / (tp + fp), 2)
-        recall = round(tp / (tp + fn), 2)
-        F1 = round(2 * ((precision * recall) / (precision + recall)), 2)
+        if type_up == self.evaluation_type and (self.evaluation_type == "date" or self.evaluation_type == "person"):
+            precision = float('nan')
+            recall = float('nan')
+            F1 = float('nan')
+        else:
+            # precision (P): TP / (TP + FP)
+            # Recall (R): TP / (TP + FN)
+            # F1 score: 2 * ((P * R) / (P + R))
+            precision = round(tp / (tp + fp), 2)
+            recall = round(tp / (tp + fn), 2)
+            if precision + recall == 0:
+                F1 = 0
+            else:
+                F1 = round(2 * ((precision * recall) / (precision + recall)), 2)
         # print("F1 {}".format(F1))
         # print(type_up,type_lower)
         # print("tp: {},fp: {},fn: {}".format(tp,fp,fn))
@@ -219,11 +257,11 @@ class DateEvaluation(Evaluation):
 
 class PersonNameEvaluation(Evaluation):
     evaluation_type = "person"
-    annotation = "person_type"
+    annotation = "personType"
     col = "person_name_annotations"
 
 
 class PhysicalAddressEvaluation(Evaluation):
     evaluation_type = "address"
-    annotation = "address_type"
+    annotation = "addressType"
     col = "physical_location_annotations"
